@@ -6,7 +6,7 @@ from persisty.attr.generator.attr_value_generator_abc import AttrValueGeneratorA
 @dataclass
 class StrSequenceGenerator(AttrValueGeneratorABC):
     """
-    Sequence Id Generator. Note: There could be a security concern with making this optional for create operations - for
+    Sequence id generator. Note: There could be a security concern with making this optional for create operations - for
     filtered storage, it could open a way for attackers to check if an id exists. (By trying to create it)
     Typically sql based keys will generate this value in the database rather than relying on a client.
     """
@@ -16,7 +16,7 @@ class StrSequenceGenerator(AttrValueGeneratorABC):
     value: int = 1
     step: int = 1
 
-    def transform(self, value):
+    def transform(self, value, item):
         if self.always or not value:
             value = self.format.format(value=self.value)
             self.value += self.step
