@@ -5,8 +5,7 @@ from persisty.util.undefined import UNDEFINED
 
 
 class TimestampGenerator(AttrValueGeneratorABC):
-    def transform(self, value):
+    def transform(self, value, item):
         if value is UNDEFINED:
             return datetime.now().astimezone(timezone.utc)
-        else:
-            return value
+        return value
