@@ -24,8 +24,7 @@ class BatchEdit(Generic[C, U]):
             return key_config.to_key_str(self.create_item)
         if self.update_item:
             return key_config.to_key_str(self.update_item)
-        else:
-            return self.delete_key
+        return self.delete_key
 
 
 def batch_edit_dataclass_for(
@@ -37,9 +36,11 @@ def batch_edit_dataclass_for(
             "update_item": Optional[update_input_type],
             "delete_key": Optional[str],
         },
+        "__doc__": f"Batch edit of {type_name}",
         "create_item": None,
         "update_item": None,
         "delete_key": None,
     }
+    # noinspection PyTypeChecker
     type_ = dataclass(type(type_name, (), params))
     return type_
