@@ -1,5 +1,6 @@
 import json
 from datetime import timezone
+from enum import Enum
 from typing import Optional, List, Iterator, Tuple, Any, Dict
 
 from dataclasses import dataclass
@@ -7,6 +8,7 @@ from uuid import UUID
 
 import marshy
 from marshy.types import ExternalItemType
+from sqlalchemy import Table, and_, select, func, Column
 from sqlalchemy.engine import Engine
 from sqlalchemy.exc import DatabaseError
 from sqlalchemy.sql.elements import BindParameter, or_
@@ -26,9 +28,6 @@ from persisty.search_filter.search_filter_abc import SearchFilterABC
 from persisty.search_order.search_order import SearchOrder
 from persisty.store.store_abc import StoreABC, T
 from persisty.store_meta import StoreMeta
-
-from sqlalchemy import Table, and_, select, func, Column
-
 from persisty.util import UNDEFINED, from_base64, to_base64
 
 
@@ -37,7 +36,7 @@ def catch_db_error(fn):
         try:
             return fn(*args, **kwargs)
         except DatabaseError as e:
-            raise PersistyError(e)
+            raise PersistyError(e) from e
 
     return wrapper
 
@@ -106,6 +105,7 @@ class SqlalchemyTableStore(StoreABC):
         items = [self._load_row(r) for r in results]
         return items
 
+    # pylint: disable=W0613
     @catch_db_error
     def _update(
         self,
@@ -115,7 +115,7 @@ class SqlalchemyTableStore(StoreABC):
         search_filter: SearchFilterABC = INCLUDE_ALL,
     ) -> Optional[T]:
         with self.engine.begin() as connection:
-            stmt = self.table.update(self._key_where_clause_from_item(updates))
+            stmt = self.table.update().where(self._key_where_clause_from_item(updates))
             updates = self._dump(updates, True)
             connection.execute(stmt, updates)
             key_dict = self.meta.key_config.to_key_dict(key)
@@ -127,7 +127,7 @@ class SqlalchemyTableStore(StoreABC):
     def delete(self, key: str) -> bool:
         with self.engine.begin() as connection:
             key = self.meta.key_config.to_key_dict(key)
-            stmt = self.table.delete(whereclause=self._key_where_clause())
+            stmt = self.table.delete().where(self._key_where_clause())
             result = connection.execute(stmt, key)
             connection.commit()
             return bool(result.rowcount)
@@ -144,13 +144,15 @@ class SqlalchemyTableStore(StoreABC):
         if not handled:
             count = sum(1 for _ in self.search_all(search_filter))
             return count
-        stmt = select([func.count()]).select_from(self.table)
+        stmt = func.count().select()
+        stmt = stmt.select_from(self.table)
         if where_clause is not None:
             stmt = stmt.where(where_clause)
         with self.engine.begin() as connection:
             row = connection.execute(stmt).first()
             return row[0]
 
+    # pylint: disable=R0914,E1101
     @catch_db_error
     def search(
         self,
@@ -260,12 +262,13 @@ class SqlalchemyTableStore(StoreABC):
         edits: List[BatchEdit],
         results_by_id: Dict[UUID, BatchEditResult],
     ):
-        stmt = self.table.insert(bind=connection)
+        stmt = self.table.insert()  # bind=connection)
         items_to_create = [self._dump(e.create_item, False) for e in edits]
         connection.execute(stmt, items_to_create)
         for insert in edits:
             results_by_id[insert.id] = BatchEditResult(insert, True)
 
+    # pylint: disable=E1101
     def _batch_update(
         self,
         connection,
@@ -292,10 +295,12 @@ class SqlalchemyTableStore(StoreABC):
                 if attr.update_generator:
                     if attr.updatable:
                         value = attr.update_generator.transform(
-                            getattr(edit.update_item, attr.name)
+                            getattr(edit.update_item, attr.name), edit.update_item
                         )
                     else:
-                        value = attr.update_generator.transform(UNDEFINED)
+                        value = attr.update_generator.transform(
+                            UNDEFINED, edit.update_item
+                        )
                 elif attr.updatable:
                     value = getattr(edit.update_item, attr.name)
                     if value is UNDEFINED:
@@ -303,7 +308,7 @@ class SqlalchemyTableStore(StoreABC):
                 if value is not UNDEFINED:
                     setattr(item, attr.name, _transform_type(value))
             item_updates = self._dump(item, True)
-            stmt = self.table.update(self._key_where_clause_from_item(item))
+            stmt = self.table.update().where(self._key_where_clause_from_item(item))
             connection.execute(stmt, item_updates)
             results_by_id[edit.id].success = True
 
@@ -317,7 +322,7 @@ class SqlalchemyTableStore(StoreABC):
         delete_keys = [key_config.to_key_dict(d.delete_key) for d in edits]
         existing_keys = self._get_existing_keys(connection, delete_keys)
         where_clause = self._key_where_clause_from_dicts(delete_keys)
-        stmt = self.table.delete(whereclause=where_clause)
+        stmt = self.table.delete().where(where_clause)
         connection.execute(stmt)
         deleted_keys = {key_config.to_key_str(k) for k in existing_keys}
         for delete in edits:
@@ -330,7 +335,7 @@ class SqlalchemyTableStore(StoreABC):
         return existing_keys
 
     def _load_row(self, row):
-        # Row is a KeyedTuple - _asdict is to match the namedtuple API (it's not private!)
+        # Row is a KeyedTuple - as dict is to match the namedtuple API (it's not private!)
         # noinspection PyProtectedMember
         item = row._asdict()
         return self._load(item)
@@ -350,6 +355,8 @@ class SqlalchemyTableStore(StoreABC):
                 if not value.tzinfo:
                     value.replace(tzinfo=timezone.utc)
                 value = value.strftime("%Y-%m-%dT%H:%M:%S+00:00")
+            if isinstance(value, Enum):
+                value = value.name
             loaded[attr_.name] = value
         # noinspection PyTypeChecker
         result = marshy.load(self.meta.get_read_dataclass(), loaded)
@@ -364,7 +371,7 @@ class SqlalchemyTableStore(StoreABC):
             else:
                 generator = attr_.create_generator
             if generator:
-                value = generator.transform(value)
+                value = generator.transform(value, item)
             setattr(item, attr_.name, value)
             if value is UNDEFINED:
                 continue
@@ -374,13 +381,15 @@ class SqlalchemyTableStore(StoreABC):
             ):
                 value = json.dumps(value)
             dumped[attr_.name] = _transform_type(value)
-
+        if is_update:
+            for attr_name in self.meta.key_config.get_key_attrs():
+                dumped[f"{attr_name}_1"] = dumped[attr_name]
         return dumped
 
     def _key_where_clause(self):
         key_where_clause = None
         for attr_name in self.meta.key_config.get_key_attrs():
-            # exp = self.table.columns.get(attr_.name) == f':{attr_.name}'
+            # pylint: disable=E1101
             exp = self.table.columns.get(attr_name) == BindParameter(attr_name)
             if key_where_clause:
                 key_where_clause &= exp
@@ -401,6 +410,7 @@ class SqlalchemyTableStore(StoreABC):
         return where_clause
 
     def _key_where_clause_from_dict(self, item: ExternalItemType):
+        # pylint: disable=E1101
         where_clause = and_(
             self.table.columns.get(attr_name) == item.get(attr_name)
             for attr_name in self.meta.key_config.get_key_attrs()
@@ -439,6 +449,7 @@ class SqlalchemyTableStore(StoreABC):
         search_order.validate_for_attrs(self.meta.attrs)
         orders = []
         for order_attr in search_order.orders:
+            # pylint: disable=E1101
             order = self.table.columns.get(order_attr.attr)
             if order_attr.desc:
                 order = order.desc()
@@ -446,6 +457,7 @@ class SqlalchemyTableStore(StoreABC):
         return orders
 
     def _default_order_by(self) -> List[Column]:
+        # pylint: disable=E1101
         orders = [
             self.table.columns.get(attr_name)
             for attr_name in self.meta.key_config.get_key_attrs()
