@@ -18,10 +18,15 @@ def get_seed_items(store_meta: StoreMeta) -> Iterable[T]:
         item_dataclass = store_meta.get_stored_dataclass()
         seed_items = []
         for data in seed_data:
+            # noinspection PyTypeChecker
             item = marshy.load(item_dataclass, data)
             for attr in store_meta.attrs:
                 if attr.create_generator and getattr(item, attr.name) is UNDEFINED:
-                    setattr(item, attr.name, attr.create_generator.transform(UNDEFINED))
+                    setattr(
+                        item,
+                        attr.name,
+                        attr.create_generator.transform(UNDEFINED, item),
+                    )
             seed_items.append(item)
         return seed_items
     except Exception as e:
@@ -35,6 +40,9 @@ def get_seed_data(name: str) -> List[ExternalItemType]:
     items = getattr(seeds, name, None)
     if items is not None:
         return items
-    seed_data = resources.read_text(seeds_package, name + ".json")
+    try:
+        seed_data = resources.read_text(seeds_package, name + ".json")
+    except FileNotFoundError:
+        return []
     seed_data = json.loads(seed_data)
     return seed_data
