@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from typing import Optional, TypeVar, Generic, Iterator, FrozenSet
+from typing import Optional, TypeVar, Generic, FrozenSet
 
 from marshy.types import ExternalItemType
 
@@ -7,7 +7,7 @@ T = TypeVar("T")
 
 
 class KeyConfigABC(ABC, Generic[T]):
-    """ Object for extracting / inserting string keys into stored items. """
+    """Object for extracting / inserting string keys into stored items."""
 
     @abstractmethod
     def to_key_str(self, item: T) -> str:
