@@ -1,29 +1,29 @@
 from dataclasses import dataclass
-from typing import Optional, ForwardRef, List, Dict
+from typing import Dict, ForwardRef, Generic, List, Optional, TypeVar
 
+import marshy
 from marshy.types import ExternalItemType
 from schemey import schema_from_type
-from schemey.schema import str_schema
 from servey.security.authorization import Authorization
 
 from persisty.attr.attr import Attr, DEFAULT_PERMITTED_FILTER_OPS
 from persisty.attr.attr_type import AttrType
-from persisty.factory.store_factory_abc import StoreFactoryABC
+from persisty.attr.generator.default_value_generator import DefaultValueGenerator
+from persisty.link.inbound_link import InboundLink
 from persisty.link.linked_store_abc import LinkedStoreABC
 from persisty.link.on_delete import OnDelete
-
-from typing import Generic, TypeVar
+from persisty.store_meta import StoreMeta
 
 T = TypeVar("T")
 
 
 class BelongsToCallable(Generic[T]):
-    def __init__(self, key: str, store_factory: StoreFactoryABC):
+    def __init__(self, key: str, store_meta: StoreMeta):
         self.key = key
-        self.store_factory = store_factory
+        self.store_meta = store_meta
 
     def __call__(self, authorization: Optional[Authorization] = None) -> Optional[T]:
-        store = self.store_factory.create(authorization)
+        store = self.store_meta.create_secured_store(authorization)
         item = store.read(self.key)
         return item
 
@@ -33,7 +33,7 @@ class BelongsTo(LinkedStoreABC, Generic[T]):
     name: Optional[str] = None
     key_attr_name: Optional[str] = None
     optional: Optional[bool] = None
-    on_delete: OnDelete = OnDelete.BLOCK
+    on_delete: OnDelete = OnDelete.IGNORE
 
     def get_name(self) -> str:
         return self.name
@@ -58,11 +58,13 @@ class BelongsTo(LinkedStoreABC, Generic[T]):
         self.name = name
         if self.key_attr_name is None:
             self.key_attr_name = f"{name}_id"
+        if self.linked_store_type is None:
+            self.linked_store_type = owner.__dict__["__annotations__"][name]
 
     def __get__(self, obj, obj_type) -> BelongsToCallable[T]:
         return BelongsToCallable(
             key=getattr(obj, self.key_attr_name),
-            store_factory=self.get_linked_store_factory(),
+            store_meta=self.get_linked_store_meta(),
         )
 
     def update_attrs(self, attrs_by_name: Dict[str, Attr]):
@@ -70,17 +72,24 @@ class BelongsTo(LinkedStoreABC, Generic[T]):
             return
         type_ = Optional[str] if self.optional else str
         schema = schema_from_type(type_)
+        create_generator = DefaultValueGenerator(None) if self.optional else None
         attrs_by_name[self.key_attr_name] = Attr(
             self.key_attr_name,
             AttrType.STR,
             schema,
             sortable=False,
+            create_generator=create_generator,
             permitted_filter_ops=DEFAULT_PERMITTED_FILTER_OPS,
         )
 
     def update_json_schema(self, json_schema: ExternalItemType):
         id_attr_schema = json_schema.get("properties").get(self.key_attr_name)
-        id_attr_schema["persistyBelongsTo"] = self.linked_store_name
+        if not id_attr_schema:
+            return
+        id_attr_schema["persistyBelongsTo"] = {
+            "linked_store_name": self.get_linked_store_name(),
+            "on_delete": marshy.dump(self.on_delete),
+        }
 
-        id_attr_schema = json_schema.get("properties").get(self.key_attr_name)
-        id_attr_schema["persistyBelongsTo"] = self.linked_store_name
+    def get_inbound_links(self, store_meta: StoreMeta) -> List[InboundLink]:
+        return [InboundLink(store_meta, self.key_attr_name, self.on_delete)]
