@@ -11,14 +11,19 @@ class ResultSet(Generic[T]):
     next_page_key: str = None
 
 
-def result_set_dataclass_for(type_: Type[T]) -> Type[ResultSet[T]]:
+def result_set_dataclass_for(
+    type_: Type[T], type_name: Optional[str] = None
+) -> Type[ResultSet[T]]:
     params = {
         "__annotations__": {
             "results": List[type_],
             "next_page_key": Optional[str],
         },
+        "__doc__": f"Result Set of {type_.__name__}",
         "next_page_key": None,
     }
-    type_name = f"{type_.__name__}ResultSet"
+    if not type_name:
+        type_name = f"{type_.__name__}ResultSet"
+    # noinspection PyTypeChecker
     type_ = dataclass(type(type_name, (ResultSet,), params))
     return type_
