@@ -11,13 +11,14 @@ from persisty.search_filter.search_filter_abc import SearchFilterABC, T
 class Not(SearchFilterABC[T]):
     search_filter: SearchFilterABC
 
+    # pylint: disable=W0221
     def __new__(cls, search_filter: SearchFilterABC):
         """Strip out direct nested Not"""
         if isinstance(search_filter, Not):
             return search_filter.search_filter
-        elif search_filter is EXCLUDE_ALL:
+        if search_filter is EXCLUDE_ALL:
             return INCLUDE_ALL
-        elif search_filter is INCLUDE_ALL:
+        if search_filter is INCLUDE_ALL:
             return EXCLUDE_ALL
         return super(Not, cls).__new__(cls)
 
@@ -25,5 +26,5 @@ class Not(SearchFilterABC[T]):
         result = Not(self.search_filter.lock_attrs(attrs))
         return result
 
-    def match(self, value: T, attrs: Tuple[Attr, ...]) -> bool:
-        return not self.search_filter.match(value, attrs)
+    def match(self, item: T, attrs: Tuple[Attr, ...]) -> bool:
+        return not self.search_filter.match(item, attrs)
