@@ -1,4 +1,4 @@
-from typing import Any, Tuple, Iterator, Generic, TypeVar
+from typing import Any, Tuple, Generic, TypeVar, List
 
 from dataclasses import dataclass
 
@@ -19,7 +19,7 @@ class SearchOrderAttr(Generic[T]):
                 return
         raise ValueError(f"search_order_invalid:{self.attr}")
 
-    def sort(self, items: Iterator[T]) -> Iterator[T]:
+    def sort(self, items: List[T]) -> List[T]:
         items = sorted(items, key=self.key, reverse=self.desc)
         return items
 
@@ -27,14 +27,12 @@ class SearchOrderAttr(Generic[T]):
         value = getattr(item, self.attr, UNDEFINED)
         if value in (None, UNDEFINED):
             return not self.desc, ""
-        else:
-            return self.desc, value
+        return self.desc, value
 
     def lt(self, a: T, b: T) -> bool:
         if self.desc:
             return self.key(a) > self.key(b)
-        else:
-            return self.key(a) < self.key(b)
+        return self.key(a) < self.key(b)
 
     def eq(self, a: T, b: T) -> bool:
         return self.key(a) == self.key(b)
