@@ -33,9 +33,11 @@ def search_order_dataclass_for(
             "attr": Optional[sortable_attrs],
             "desc": bool,
         },
+        "__doc__": f"Search Order for {store_meta.name}",
         "attr": None,
         "desc": False,
     }
     name = f"{to_camel_case(store_meta.name)}SearchOrder"
+    # noinspection PyTypeChecker
     type_ = dataclass(type(name, (SearchOrderFactoryABC,), params))
     return type_
