@@ -1,6 +1,5 @@
 import dataclasses
 import inspect
-from inspect import Signature
 from typing import Type, Optional, List
 
 from servey.action.action import Action, action, get_action
@@ -10,8 +9,8 @@ from servey.trigger.web_trigger import WebTrigger, WebTriggerMethod
 
 from persisty.batch_edit import batch_edit_dataclass_for
 from persisty.batch_edit_result import batch_edit_result_dataclass_for
-from persisty.factory.store_factory_abc import StoreFactoryABC
 from persisty.link.link_abc import LinkABC
+from persisty.result import to_result
 from persisty.result_set import result_set_dataclass_for
 from persisty.search_filter.include_all import INCLUDE_ALL
 from persisty.search_filter.search_filter_factory import SearchFilterFactoryABC
@@ -21,12 +20,10 @@ from persisty.store_meta import get_meta, StoreMeta
 
 
 def action_for_create(
-    store_factory: StoreFactoryABC,
-    item_type: Type,
+    store_meta: StoreMeta,
+    result_type: Type,
     create_input_type: Type,
 ) -> Action:
-    store_meta = store_factory.get_meta()
-
     @action(
         name=f"{store_meta.name}_create",
         description=f"Create and return an item in {store_meta.name}",
@@ -36,17 +33,16 @@ def action_for_create(
     )
     def create(
         item: create_input_type, authorization: Optional[Authorization] = None
-    ) -> Optional[item_type]:
-        store = store_factory.create(authorization)
-        created = store.create(item)
-        return created
+    ) -> Optional[result_type]:
+        secured_store = store_meta.create_secured_store(authorization)
+        created = secured_store.create(item)
+        result = to_result(created, secured_store.get_meta())
+        return result
 
     return get_action(create)
 
 
-def action_for_read(store_factory: StoreFactoryABC, item_type: Type) -> Action:
-    store_meta = store_factory.get_meta()
-
+def action_for_read(store_meta: StoreMeta, result_type: Type) -> Action:
     @action(
         name=f"{store_meta.name}_read",
         description=f"Read an item from {store_meta.name} given a key",
@@ -60,22 +56,21 @@ def action_for_read(store_factory: StoreFactoryABC, item_type: Type) -> Action:
     )
     def read(
         key: str, authorization: Optional[Authorization] = None
-    ) -> Optional[item_type]:
-        store = store_factory.create(authorization)
-        result = store.read(key)
+    ) -> Optional[result_type]:
+        secured_store = store_meta.create_secured_store(authorization)
+        item = secured_store.read(key)
+        result = to_result(item, secured_store.get_meta())
         return result
 
     return get_action(read)
 
 
 def action_for_update(
-    store_factory: StoreFactoryABC,
-    item_type: Type,
+    store_meta: StoreMeta,
+    result_type: Type,
     update_input_type: Type,
     search_filter_type: Type[SearchFilterFactoryABC],
 ) -> Action:
-    store_meta = store_factory.get_meta()
-
     @action(
         name=f"{store_meta.name}_update",
         description=f"Update and return an item in {store_meta.name}",
@@ -91,19 +86,18 @@ def action_for_update(
         item: update_input_type,
         precondition: Optional[search_filter_type] = None,
         authorization: Optional[Authorization] = None,
-    ) -> Optional[item_type]:
+    ) -> Optional[result_type]:
         store_meta.key_config.from_key_str(key, item)
-        store = store_factory.create(authorization)
-        search_filter = precondition.to_search_filter() if precondition else INCLUDE_ALL
-        updated = store.update(item, search_filter)
-        return updated
+        secured_store = store_meta.create_secured_store(authorization)
+        search_filter = _create_search_filter(search_filter_type, precondition)
+        updated = secured_store.update(item, search_filter)
+        result = to_result(updated, secured_store.get_meta())
+        return result
 
     return get_action(update)
 
 
-def action_for_delete(store_factory: StoreFactoryABC) -> Action:
-    store_meta = store_factory.get_meta()
-
+def action_for_delete(store_meta: StoreMeta) -> Action:
     @action(
         name=f"{store_meta.name}_delete",
         description=f"Delete an item in {store_meta.name}",
@@ -115,23 +109,23 @@ def action_for_delete(store_factory: StoreFactoryABC) -> Action:
         ),
     )
     def delete(key: str, authorization: Optional[Authorization] = None) -> bool:
-
-        store = store_factory.create(authorization)
-        result = store.delete(key)
+        secured_store = store_meta.create_secured_store(authorization)
+        result = secured_store.delete(key)
         return result
 
     return get_action(delete)
 
 
 def action_for_search(
-    store_factory: StoreFactoryABC,
-    item_type: Type,
+    store_meta: StoreMeta,
+    result_type: Type,
     search_filter_type: Optional[Type[SearchFilterFactoryABC]],
     search_order_type: Type[SearchOrderFactoryABC],
 ) -> Action:
-    store_meta = store_factory.get_meta()
+    item_name = store_meta.name.title().replace("_", "")
+    result_set_name = f"{item_name}ResultSet"
     # noinspection PyTypeChecker
-    result_set_type = result_set_dataclass_for(item_type)
+    result_set_type = result_set_dataclass_for(result_type, result_set_name)
     setattr(generated, result_set_type.__name__, result_set_type)
 
     @action(
@@ -151,10 +145,8 @@ def action_for_search(
         limit: Optional[int] = None,
         authorization: Optional[Authorization] = None,
     ) -> result_set_type:
-        store = store_factory.create(authorization)
-        search_filter = (
-            search_filter.to_search_filter() if search_filter else INCLUDE_ALL
-        )
+        secured_store = store_meta.create_secured_store(authorization)
+        search_filter = _create_search_filter(search_filter_type, search_filter)
         if search_order:
             # noinspection PyArgumentList,PyDataclass
             search_order = search_order_type(
@@ -164,18 +156,22 @@ def action_for_search(
                 }
             )
             search_order = search_order.to_search_order()
-        result_set = store.search(search_filter, search_order, page_key, limit)
+        result_set = secured_store.search(search_filter, search_order, page_key, limit)
 
         # noinspection PyArgumentList
         result_set = result_set_type(
-            results=result_set.results,
+            results=[
+                to_result(item, secured_store.get_meta()) for item in result_set.results
+            ],
             next_page_key=result_set.next_page_key,
         )
         return result_set
 
     if search_order_type is None:
         sig = inspect.signature(search)
-        sig = sig.replace(parameters=[p for p in sig.parameters.values() if p.name != 'search_order'])
+        sig = sig.replace(
+            parameters=[p for p in sig.parameters.values() if p.name != "search_order"]
+        )
         search.__signature__ = sig
 
     search = action(
@@ -194,11 +190,9 @@ def action_for_search(
 
 
 def action_for_count(
-    store_factory: StoreFactoryABC,
+    store_meta: StoreMeta,
     search_filter_type: Type[SearchFilterFactoryABC],
 ) -> Action:
-    store_meta = store_factory.get_meta()
-
     @action(
         name=f"{store_meta.name}_count",
         description=f"Get a count from {store_meta.name}",
@@ -213,19 +207,15 @@ def action_for_count(
         search_filter: Optional[search_filter_type] = None,
         authorization: Optional[Authorization] = None,
     ) -> int:
-        store = store_factory.create(authorization)
-        search_filter = (
-            search_filter.to_search_filter() if search_filter else INCLUDE_ALL
-        )
-        result = store.count(search_filter)
+        secured_store = store_meta.create_secured_store(authorization)
+        search_filter = _create_search_filter(search_filter_type, search_filter)
+        result = secured_store.count(search_filter)
         return result
 
     return get_action(count)
 
 
-def action_for_read_batch(store_factory: StoreFactoryABC, item_type: Type) -> Action:
-    store_meta = store_factory.get_meta()
-
+def action_for_read_batch(store_meta: StoreMeta, result_type: Type) -> Action:
     @action(
         name=f"{store_meta.name}_read_batch",
         description=f"Read a batch of items from {store_meta.name} given keys",
@@ -237,20 +227,20 @@ def action_for_read_batch(store_factory: StoreFactoryABC, item_type: Type) -> Ac
     )
     def read_batch(
         keys: List[str], authorization: Optional[Authorization] = None
-    ) -> List[Optional[item_type]]:
-        store = store_factory.create(authorization)
-        results = store.read_batch(keys)
+    ) -> List[Optional[result_type]]:
+        secured_store = store_meta.create_secured_store(authorization)
+        items = secured_store.read_batch(keys)
+        results = [to_result(item, secured_store.get_meta()) for item in items]
         return results
 
     return get_action(read_batch)
 
 
 def action_for_edit_batch(
-    store_factory: StoreFactoryABC,
+    store_meta: StoreMeta,
     create_input_type: Type,
     update_input_type: Type,
 ) -> Action:
-    store_meta = store_factory.get_meta()
     batch_edit_type = batch_edit_dataclass_for(
         store_meta.name.title() + "BatchEdit", create_input_type, update_input_type
     )
@@ -267,8 +257,8 @@ def action_for_edit_batch(
     def edit_batch(
         edits: List[batch_edit_type], authorization: Optional[Authorization] = None
     ) -> List[batch_edit_result_type]:
-        store = store_factory.create(authorization)
-        results = store.edit_batch(edits)
+        secured_store = store_meta.create_secured_store(authorization)
+        results = secured_store.edit_batch(edits)
         results = [
             batch_edit_result_type(
                 edit=edit,
@@ -290,6 +280,8 @@ def wrap_links_in_actions(read_type: Type):
         if isinstance(v, LinkABC):
             overrides[k] = _to_action_fn(meta, v)
     if overrides:
+        overrides["__doc__"] = read_type.__dict__.get("__doc__")
+        # noinspection PyTypeChecker
         read_type = dataclasses.dataclass(
             type(read_type.__name__, (read_type,), overrides)
         )
@@ -300,6 +292,7 @@ def wrap_links_in_actions(read_type: Type):
 def _to_action_fn(meta: StoreMeta, link: LinkABC):
     return_type = link.get_linked_type("persisty.servey.generated")
 
+    # pylint: disable=C2801
     def wrapper(self, authorization: Optional[Authorization] = None) -> return_type:
         fn = link
         if hasattr(link, "__get__"):
@@ -328,3 +321,17 @@ def _to_action_fn(meta: StoreMeta, link: LinkABC):
     )
 
     return wrapped
+
+
+def _create_search_filter(search_filter_factory_type, search_filter_input):
+    # Since this may be from graphql, it may not actually be the class we expect, so we
+    # replace the instance
+    if not search_filter_input:
+        return INCLUDE_ALL
+    kwargs = {
+        f.name: getattr(search_filter_input, f.name)
+        for f in dataclasses.fields(search_filter_input)
+    }
+    search_filter_factory = search_filter_factory_type(**kwargs)
+    search_filter = search_filter_factory.to_search_filter()
+    return search_filter
